"""
TexisXMLRequest Module

See README.txt for copyright and license information.
"""
# 	$Id: TexisXMLRequest.py,v 1.9 2004/05/10 16:36:58 davidc Exp $	

from OFS.SimpleItem import SimpleItem
from Globals import DTMLFile
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from xml.sax import make_parser
from xml.sax.handler import ContentHandler
from AccessControl import ClassSecurityInfo

security = ClassSecurityInfo()

manage_addTexisXMLRequestForm = DTMLFile("dtml/addTexisXMLRequestForm", globals())

def manage_addTexisXMLRequest(self, id, title, url, REQUEST=None):
	"Adds a Texis XML Request object to a folder"
	newTexisXMLRequest = TexisXMLRequest(id, title, url)
	self._setObject(id, newTexisXMLRequest)
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)


class TexisXMLRequest(SimpleItem):
	"""A Texis XML request object"""

	meta_type = 'Texis XML Request'

	manage_options = ({'label':'Edit',
					   'action':'manage_main'},
					  {'label':'Test',
					   'action':'manage_test'}
					  )

	manage_test = PageTemplateFile("zpt/testTexisXMLRequest", globals())

	manage_main = PageTemplateFile("zpt/mainTexisXMLRequest", globals())

	index_html = PageTemplateFile("zpt/indexTexisXMLRequest", globals())

	def __init__(self, id, title, url):
		self.id = id
		self.title = title
		self.url = url				

	security.declareProtected('Add Texis XML Request', 'editTexisXMLRequest')
	def editTexisXMLRequest(self, title, url, REQUEST=None):
		"A method to edit Texis XML Request properties"
		self.title = title
		self.url = url
		if REQUEST is not None:
			return self.manage_main(self, REQUEST)		

	security.declarePublic('getResults')
	def getResults(self, REQUEST=None, **kwargs):
		"""The main method - returns Python list of result rows.
		   Each row is dictionary of column names and row values.
		   Pass Texis parameters as keyword arguments.
		"""
		from ZTUtils import make_query, url_query
		qs = ''                                             # query string passed to Texis
		if REQUEST is not None:
			qs = url_query(REQUEST, omit=kwargs.keys()) # remove arguments from query string
			qs = qs.split('?')[1]                       
		if kwargs:                                          # add kwargs to qs
			qs = (qs and (qs + '&')) + make_query(kwargs)
		texis_request = self.url + (qs and ('?' + qs))      # construct Texis request URL
		parser = make_parser()
		handler = TexisXMLHandler()
		parser.setContentHandler(handler)
		try:                                                # send request/parse results
			parser.parse(texis_request)
		except:
			# XXX would like to return Texis errors,
			# but this is safe for the moment
			return None
		else:
			return handler.results

	__call__ = getResults

			
class TexisXMLHandler(ContentHandler):
	# the SAX handler

	def __init__(self):
		# should probably set these values to None
		self.results = []
		self.result = {}
		self.column = ''
		self.row = ''

	def startElement(self, name, attrs):
		# need to deal with errors
		if name == 'results':
			self.results = []		# initialize results
		elif name == 'result':
			self.result = {}		# reset result
		else:
			self.column = name	# set new column name
			# not sure if characters() is invoked for empty elements
			# so explicitly set to empty string
			self.row = ''		# reset row value

	def characters(self, chars):
		self.row += chars			# append chars to value

	def endElement(self, name):
		# need to deal with errors
		if name == 'result':
			self.results.append(self.result)
		elif name != 'results':
			self.result[self.column] = self.row
